@@ -1,6 +1,7 @@
 # vim:ts=4 sw=4 expandtab softtabstop=4
 import unittest
 import warnings
+import sys
 
 from collections import OrderedDict
 import jsonmerge
@@ -91,16 +92,14 @@ class TestMerge(unittest.TestCase):
         with self.assertRaises(SchemaError) as cm:
             merger.merge(None, "a", merge_options={'version': {'metadata': 'foo'}})
 
+    @unittest.skipIf(sys.version_info < (3,2), "not supported on Python <3.2")
     def test_version_meta_deprecated(self):
         schema = {'mergeStrategy': 'version'}
         merger = jsonmerge.Merger(schema)
 
-        with warnings.catch_warnings(record=True) as w:
+        with self.assertWarnsRegex(DeprecationWarning, r"'meta' argument is deprecated"):
             base = merger.merge(None, 'a', meta={'foo': 'bar'})
 
-            self.assertEqual(len(w), 1)
-            self.assertTrue(issubclass(w[0].category, DeprecationWarning))
-
     def test_version_ignoredups_false(self):
 
         schema = {'mergeStrategy': 'version',
@@ -195,6 +194,55 @@ class TestMerge(unittest.TestCase):
 
         self.assertEqual(base, ["a", "b"])
 
+    def test_append_with_sort(self):
+        schema = {'mergeStrategy': 'append', 
+                  'mergeOptions': 
+                  { 'sortByRef': 'name'}}
+
+        base = None
+        head = [{"name": "b"}, {"name": "a"}]
+        base = jsonmerge.merge(base, head , schema)
+        
+        self.assertEqual(base, [{"name": "a"}, {"name": "b"}])
+
+    def test_append_with_sort_reversed(self):
+        schema = {'mergeStrategy': 'append',
+                  'mergeOptions':
+                  { 'sortByRef': '/',
+                    'sortReverse': True }}
+
+        base = ['a', 'c']
+        head = ['b']
+        base = jsonmerge.merge(base, head, schema)
+
+        self.assertEqual(base, ['c', 'b', 'a'])
+
+    def test_append_with_sort_invalid_ref(self):
+        schema = {'mergeStrategy': 'append',
+                  'mergeOptions': { 'sortByRef': 'name'}}
+
+        base = [
+                {"item": "a", "name": "a"},
+                {"item": "c", "name": "c"},
+        ]
+
+        head = [
+                {"item": "d"},
+                {"item": "b", "name": "b"},
+                {"item": "f"},
+        ]
+
+        base = jsonmerge.merge(base, head, schema)
+
+        self.assertEqual(base, [
+                {"item": "a", "name": "a"},
+                {"item": "b", "name": "b"},
+                {"item": "c", "name": "c"},
+                {"item": "d"},
+                {"item": "f"},
+        ]
+        )
+
     def test_append_type_error(self):
 
         schema = {'mergeStrategy': 'append'}
@@ -225,6 +273,20 @@ class TestMerge(unittest.TestCase):
 
         self.assertEqual(base, {'a': "a", 'b': "b"})
 
+    def test_merge_default_type_mismatch(self):
+
+        schema = {}
+
+        base = { "foo": "" }
+        head = { "foo": {} }
+
+        # default_strategy() looks at the type in head to select the strategy. So here
+        # it selects objectMerge, but base is a string. Hence a BaseInstanceError is
+        # raised.
+
+        with self.assertRaises(BaseInstanceError) as cm:
+            result = jsonmerge.merge(base, head, schema)
+
     def test_merge_empty_schema(self):
 
         schema = {}
@@ -1741,16 +1803,14 @@ class TestGetSchema(unittest.TestCase):
                              }
                          })
 
+    @unittest.skipIf(sys.version_info < (3,2), "not supported on Python <3.2")
     def test_version_meta_deprecated(self):
         schema = {'mergeStrategy': 'version'}
         merger = jsonmerge.Merger(schema)
 
-        with warnings.catch_warnings(record=True) as w:
+        with self.assertWarnsRegex(DeprecationWarning, r"'meta' argument is deprecated"):
             merger.get_schema(meta={'foo': 'bar'})
 
-            self.assertEqual(len(w), 1)
-            self.assertTrue(issubclass(w[0].category, DeprecationWarning))
-
     def test_version_meta_in_schema(self):
         schema = {
                 'type': 'object',
@@ -1970,8 +2030,7 @@ class TestGetSchema(unittest.TestCase):
         schema = {'mergeStrategy': 'version'}
 
         meta_schema = {
-            'id': 'http://example.com/schema_1.json',
-            '$ref': 'schema_2.json#/definitions/meta'
+            '$ref': 'http://example.com/schema_2.json#/definitions/meta'
         }
 
         schema_2 = {
@@ -2331,6 +2390,55 @@ class TestGetSchema(unittest.TestCase):
 
         self.assertEqual(schema2, expected)
 
+    def test_merge_by_id_with_sort(self):
+        schema = {
+            "properties": {
+                "awards": {
+                    "type": "array",
+                    "mergeStrategy": "arrayMergeById",
+                    'mergeOptions': 
+                        { 'sortByRef': 'id'},
+                    "items": {
+                        "properties": {
+                            "id": {"type": "string"},
+                            "field": {"type": "number"},
+                        }
+                    }
+                }
+            }
+        }
+
+        a = {
+            "awards": [
+                {"id": "B", "field": 3},
+                {"id": "A", "field": 2}
+            ]
+        }
+
+        b = {
+            "awards": [
+                {"id": "C", "field": 4},
+                {"id": "B", "field": 3}
+            ]
+        }
+
+        expected = {
+            "awards": [
+                {"id": "A", "field": 2},
+                {"id": "B", "field": 3},
+                {"id": "C", "field": 4}
+            ]
+        }
+
+        merger = jsonmerge.Merger(schema)
+
+        base = None
+        base = merger.merge(base, a)
+        base = merger.merge(base, b)
+
+        self.assertEqual(base, expected)
+
+
     def test_merge_append_additional(self):
 
         schema = {'mergeStrategy': 'objectMerge',
